@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2016-2019 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2016 -- leonerd@leonerd.org.uk
 
 package Syntax::Keyword::Try;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.11_002';
+our $VERSION = '0.01';
 
 use Carp;
 
@@ -21,7 +21,7 @@ C<Syntax::Keyword::Try> - a C<try/catch/finally> syntax for perl
 
 =head1 SYNOPSIS
 
- use Syntax::Keyword::Try;
+ use Syntax::Keyword::Try 'try';
 
  sub foo
  {
@@ -42,10 +42,10 @@ semantics in a form familiar to users of other languages, being built on a
 block labeled with the C<try> keyword, followed by at least one of a C<catch>
 or C<finally> block.
 
-As well as providing a handy syntax for this useful behaviour, this module
-also serves to contain a number of code examples for how to implement parser
-plugins and manipulate optrees to provide new syntax and behaviours for perl
-code.
+As well as being providing a handy syntax for this useful behaviour, this
+module also serves to contain a number of code examples for how to implement
+parser plugins and manipulate optrees to provide new syntax and behaviours for
+perl code.
 
 =cut
 
@@ -75,19 +75,18 @@ circumstance only the C<eval> itself would return, not the entire function.
 
 The body of a C<try {}> block may contain loop control expressions (C<redo>,
 C<next>, C<last>) which will have their usual effect on any loops that the
-C<try {}> block is contained by.
+C<try {}> block is contained by. As of the current implementation however,
+these will result in a warning
+
+ Exiting eval via redo at FILE line LINE.
+
+The use of C<no warnings 'exiting'> can avoid this.
 
 The parsing rules for the set of statements (the C<try> block and its
 associated C<catch> and C<finally>) are such that they are parsed as a self-
 contained statement. Because of this, there is no need to end with a
 terminating semicolon.
 
-Note (especially to users of L<Try::Tiny> and similar) that the C<try {}>
-block itself does not necessarily stop exceptions thrown inside it from
-propagating outside. It is the presence of a later C<catch {}> block which
-causes this to happen. A C<try> with only a C<finally> and no C<catch> will
-still propagate exceptions up to callers as normal.
-
 =head2 catch
 
    ...
@@ -95,23 +94,13 @@ still propagate exceptions up to callers as normal.
       STATEMENTS...
    }
 
-Or
-
-   ...
-   catch my $var {
-      STATEMENTS...
-   }
-
-I<Since version NEXT.>
-
-A C<catch> statement provides a block of code to the preceding C<try>
+A C<catch> statement provides a block of code to the preceeding C<try>
 statement that will be invoked in the case that the main block of code throws
 an exception. The C<catch> block can inspect the raised exception by looking
-in C<$@> in the usual way. Optionally, a new lexical variable can be
-introduced to store the exception in.
+in C<$@> in the usual way.
 
 Presence of this C<catch> statement causes any exception thrown by the
-preceding C<try> block to be non-fatal to the surrounding code. If the
+preceeding C<try> block to be non-fatal to the surrounding code. If the
 C<catch> block wishes to optionally handle some exceptions but not others, it
 can re-raise it (or another exception) by calling C<die> in the usual manner.
 
@@ -131,10 +120,10 @@ block are raised to the caller in the usual way.
       STATEMENTS...
    }
 
-A C<finally> statement provides a block of code to the preceding C<try>
+A C<finally> statement provides a block of code to the preceeding C<try>
 statement (or C<try/catch> pair) which is executed afterwards, both in the
 case of a normal execution or a thrown exception. This code block may be used
-to provide whatever clean-up operations might be required by preceding code.
+to provide whatever clean-up operations might be required by preceeding code.
 
 Because it is executed during a stack cleanup operation, a C<finally {}> block
 may not cause the containing function to return, or to alter the return value
@@ -145,52 +134,40 @@ from disturbing the value of C<$@>. If the C<finally {}> block code throws an
 exception, this will be printed as a warning and discarded, leaving C<$@>
 containing the original exception, if one existed.
 
-=head1 VALUE SEMANTICS
+=cut
 
-=over 4
+=head1 TODO
 
-B<Warning:> the feature described in this section is experimental. This
-experiment may be stablised in a later version, or may be altered or removed
-without further notice. It is present here for testing and evaluation
-purposes.
+=over 2
 
-Additionally, on I<perl> versions 5.18 and later, it will produce a warning
-in the C<experimental> category.
+=item *
+
+Value semantics. It would be nice if a C<do {}>-wrapped C<try> set could yield
+a value, in the way other similar constructs can. For example
+
+ my $x = do {
+    try { attempt(); "success" }
+    catch { "failure" }
+ };
+
+A workaround for this current lack is to wrap the C<try{} catch{}> pair in an
+anonymous function which is then immediately executed:
+
+ my $x = sub {
+    try { attempt(); return "success" }
+    catch { return "failure" }
+ }->();
+
+=item *
+
+Suppress the C<exiting> warning when using loop control systems in the
+C<try {}> block. This is sligtly nontrivial because an inplace replacement of
+an C<OP_LAST> to an C<OP_LINESEQ> containing the new hints COP isn't possible
+due to differing op types. It will require more careful re-splicing into
+parent trees.
 
 =back
 
-The syntax provided by this module may be used as a value-yielding expression.
-Because this syntax is new, experimental, and somewhat surprising, it must be
-specifically requested by name C<try_value>:
-
-   use Syntax::Keyword::Try qw( try try_value );
-
-   my $result = try do { ... } catch { ... };
-
-Also, on Perl versions 5.24 and later:
-
-   my $result = try do { ... } finally { ... };
-
-   my $result = try do { ... } catch { ... } finally { ... };
-
-Specifically, note that the expression must be spelled as C<try do { ... }> so
-that the syntax is distinct from that used by control-flow statements. The
-interposed C<do> keyword reminds the reader, and instructs the syntax parser,
-that this will be an expression, not a statement. It is not necessary to
-similarly notate the C<catch> or C<finally> blocks.
-
-In this case, the syntax behaves syntactically like an expression, and may
-appear anywhere a normal expression is allowed. It follows similar semantics
-to the purely control-flow case; if the code in the C<try> block does not
-throw an exception, then the expression as a whole yields whatever value the
-C<try> expression did. If it fails, then the C<catch> block is executed and
-the expression yields its resulting value instead. A C<finally> block, if
-present, will be evaluated for side-effects before the rest of the expression
-returns.
-
-Remember that, as in the control-flow case, the C<return> keyword will cause
-the entire containing function to return, not just the C<try> block.
-
 =cut
 
 =head1 OTHER MODULES
@@ -244,7 +221,7 @@ this ability.
 
 =head2 C<return> in a try or catch block
 
-Like L<TryCatch> and L<Syntax::Feature::Try>, the C<return> statement has its
+Like L<TryCatch> and L<Syntax::Keyword::Try>, the C<return> statement has its
 usual effect within a subroutine containing syntax provided by this module.
 Namely, it causes the containing C<sub> itself to return.
 
@@ -254,45 +231,30 @@ will only exit from the C<try> block.
 =head2 C<next>/C<last>/C<redo> in a try or catch block
 
 The loop control keywords of C<next>, C<last> and C<redo> have their usual
-effect on dynamically contained loops.
+effect on dynamically contained loops (though as noted above, in the current
+implementation a warning is produced).
 
 L<Syntax::Feature::Try> documents that these do not work there. The other
 modules make no statement either way.
 
 =head2 Value Semantics
 
-Like L<Try> and L<Syntax::Feature::Try>, the syntax provided by this module
-only works as a syntax-level statement and not an expression when the
-experimental C<try_value> feature described above has not been enabled. You
-cannot assign from the result of a C<try> block. Additionally,
-final-expression value semantics do not work, so it cannot be contained by a
-C<do> block to yield this value.
+Like L<Try> and L<Syntax::Keyword::Try>, the syntax provided by this module
+only works as a syntax-level statement and not an expression; you cannot
+assign from the result of a C<try> block. Additionally, final-expression value
+semantics do not work, so it cannot be contained by a C<do> block to yield
+this value. See above for a workaround involving an anonymous sub however.
 
 In comparison, the behaviour implemented by L<Try::Tiny> can be used as a
 valued expression, such as assigned to a variable or returned to the caller of
-its containing function. Such ability is provided by this module if the
-experimental C<try_value> feature is enabled, though it must be spelled
-differently as C<try do { ... }>.
-
-=head2 C<try> without C<catch>
-
-Like L<Syntax::Feature::Try>, the syntax provided by this module allows a
-C<try> block to be followed by only a C<finally> block, with no C<catch>. In
-this case, exceptions thrown by code contained by the C<try> are not
-suppressed, instead they propagate as normal to callers. This matches the
-behaviour familiar to Java or C++ programmers.
-
-In comparison, the code provided by L<Try> and L<Try::Tiny> always suppress
-exception propagation even without an actual C<catch> block.
-
-The L<TryCatch> module does not allow a C<try> block not followed by C<catch>.
+its containing function.
 
 =head2 Typed C<catch>
 
 Like L<Try> and L<Try::Tiny>, this module makes no attempt to perform any kind
 of typed dispatch to distinguish kinds of exception caught by C<catch> blocks.
 
-L<TryCatch> and L<Syntax::Feature::Try> both attempt to provide a kind of
+L<TryCatch> and L<Syntax::Keyword::Try> both attempt to provide a kind of
 typed dispatch where different classes of exception are caught by different
 blocks of code, or propagated up entirely to callers.
 
@@ -317,85 +279,12 @@ sub import_into
    my $class = shift;
    my ( $caller, @syms ) = @_;
 
-   @syms or @syms = qw( try );
-
    my %syms = map { $_ => 1 } @syms;
    $^H{"Syntax::Keyword::Try/try"}++ if delete $syms{try};
-   $^H{"Syntax::Keyword::Try/try_value"}++ if delete $syms{try_value};
-
-   # Ignore requests for these, as they come automatically with `try`
-   delete @syms{qw( catch finally )};
 
    croak "Unrecognised import symbols @{[ keys %syms ]}" if keys %syms;
 }
 
-=head1 WITH OTHER MODULES
-
-=head2 Future::AsyncAwait
-
-As of C<Future::AsyncAwait> version 0.10 and L<Syntax::Keyword::Try> version
-0.07, cross-module integration tests assert that basic C<try/catch> blocks
-inside an C<async sub> work correctly, including those that attempt to
-C<return> from inside C<try>.
-
-   use Future::AsyncAwait;
-   use Syntax::Keyword::Try;
-
-   async sub attempt
-   {
-      try {
-         await func();
-         return "success";
-      }
-      catch {
-         return "failed";
-      }
-   }
-
-=head1 KNOWN BUGS
-
-=head2 Thread-safety at load time cannot be assured before perl 5.16
-
-On F<perl> versions 5.16 and above this module is thread-safe.
-
-On F<perl> version 5.14 this module is thread-safe provided that it is
-C<use>d before any additional threads are created.
-
-However, when using 5.14 there is a race condition if this module is loaded
-late in the program startup, after additional threads have been created. This
-leads to the potential for it to be started up multiple times concurrently,
-which creates data races when modifying internal structures and likely leads
-to a segmentation fault, either during load or soon after when more code is
-compiled.
-
-As a workaround, for any such program that creates multiple threads, loads
-additional code (such as dynamically-discovered plugins), and has to run on
-5.14, it should make sure to
-
-   use Syntax::Keyword::Try;
-
-early on in startup, before it spins out any additional threads.
-
-(See also L<https://rt.cpan.org/Public/Bug/Display.html?id=123547>)
-
-=head2 $@ is not local'ised by C<try do> before perl 5.24
-
-On F<perl> versions 5.24 and above, or when using only control-flow statement
-syntax, C<$@> is always correctly C<local>ised.
-
-However, when using the experimental value-yielding expression version
-C<try do {...}> on perl versions 5.22 or older, the C<local>isation of C<$@>
-does not correctly apply around the expression. After such an expression, the
-value of C<$@> will leak out if a failure happened and the C<catch> block was
-invoked, overwriting any previous value that was visible there.
-
-(See also L<https://rt.cpan.org/Public/Bug/Display.html?id=124366>)
-
-=head1 ACKNOWLEDGEMENTS
-
-With thanks to C<Zefram>, C<ilmari> and others from C<irc.perl.org/#p5p> for
-assisting with trickier bits of XS logic.
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
